package javax.sound.sampled;

import java.util.ArrayList;
import com.sun.media.sound.JDK13Services;
import java.io.OutputStream;
import javax.sound.sampled.spi.AudioFileWriter;
import java.util.HashSet;
import java.io.File;
import java.net.URL;
import java.io.IOException;
import javax.sound.sampled.spi.AudioFileReader;
import java.io.InputStream;
import javax.sound.sampled.spi.FormatConversionProvider;
import java.util.Vector;
import javax.sound.sampled.spi.MixerProvider;
import java.util.List;

public class AudioSystem
{
    public static final int NOT_SPECIFIED = -1;
    
    private AudioSystem() {
    }
    
    public static Mixer.Info[] getMixerInfo() {
        List<Object> mixerInfoList = getMixerInfoList();
        return (Mixer.Info[])(Mixer.Info[])mixerInfoList.toArray(new Mixer.Info[mixerInfoList.size()]);
    }
    
    public static Mixer getMixer(Mixer.Info info) {
        List<Object> mixerProviders = getMixerProviders();
        for (int i = 0; i < mixerProviders.size(); ++i) {
            try {
                return ((MixerProvider)mixerProviders.get(i)).getMixer(info);
            }
            catch (IllegalArgumentException ex) {}
            catch (NullPointerException ex2) {}
        }
        if (info == null) {
            for (int j = 0; j < mixerProviders.size(); ++j) {
                try {
                    MixerProvider mixerProvider = (MixerProvider)mixerProviders.get(j);
                    Mixer.Info[] mixerInfo = mixerProvider.getMixerInfo();
                    int k = 0;
                    while (k < mixerInfo.length) {
                        try {
                            return mixerProvider.getMixer(mixerInfo[k]);
                        }
                        catch (IllegalArgumentException ex3) {
                            ++k;
                        }
                    }
                }
                catch (IllegalArgumentException ex4) {}
                catch (NullPointerException ex5) {}
            }
        }
        throw new IllegalArgumentException("Mixer not supported: " + ((info != null) ? info.toString() : "null"));
    }
    
    public static Line.Info[] getSourceLineInfo(Line.Info info) {
        Vector<Object> vector = new Vector();
        Mixer.Info[] mixerInfo = getMixerInfo();
        for (int i = 0; i < mixerInfo.length; ++i) {
            Line.Info[] sourceLineInfo = getMixer(mixerInfo[i]).getSourceLineInfo(info);
            for (int j = 0; j < sourceLineInfo.length; ++j) {
                vector.addElement(sourceLineInfo[j]);
            }
        }
        Line.Info[] array = new Line.Info[vector.size()];
        for (int k = 0; k < array.length; ++k) {
            array[k] = (Line.Info)vector.get(k);
        }
        return array;
    }
    
    public static Line.Info[] getTargetLineInfo(Line.Info info) {
        Vector<Object> vector = new Vector();
        Mixer.Info[] mixerInfo = getMixerInfo();
        for (int i = 0; i < mixerInfo.length; ++i) {
            Line.Info[] targetLineInfo = getMixer(mixerInfo[i]).getTargetLineInfo(info);
            for (int j = 0; j < targetLineInfo.length; ++j) {
                vector.addElement(targetLineInfo[j]);
            }
        }
        Line.Info[] array = new Line.Info[vector.size()];
        for (int k = 0; k < array.length; ++k) {
            array[k] = (Line.Info)vector.get(k);
        }
        return array;
    }
    
    public static boolean isLineSupported(Line.Info info) {
        Mixer.Info[] mixerInfo = getMixerInfo();
        int i = 0;
        while (i < mixerInfo.length) {
            if (mixerInfo[i] != null && getMixer(mixerInfo[i]).isLineSupported(info)) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    public static Line getLine(Line.Info info) throws LineUnavailableException {
        LineUnavailableException ex = null;
        List<Object> mixerProviders = getMixerProviders();
        try {
            Mixer defaultMixer = getDefaultMixer(mixerProviders, info);
            if (defaultMixer != null && defaultMixer.isLineSupported(info)) {
                return defaultMixer.getLine(info);
            }
        }
        catch (LineUnavailableException ex2) {
            ex = ex2;
        }
        catch (IllegalArgumentException ex3) {}
        for (int i = 0; i < mixerProviders.size(); ++i) {
            MixerProvider mixerProvider = (MixerProvider)mixerProviders.get(i);
            Mixer.Info[] mixerInfo = mixerProvider.getMixerInfo();
            for (int j = 0; j < mixerInfo.length; ++j) {
                try {
                    Mixer mixer = mixerProvider.getMixer(mixerInfo[j]);
                    if (isAppropriateMixer(mixer, info, true)) {
                        return mixer.getLine(info);
                    }
                }
                catch (LineUnavailableException ex4) {
                    ex = ex4;
                }
                catch (IllegalArgumentException ex5) {}
            }
        }
        for (int k = 0; k < mixerProviders.size(); ++k) {
            MixerProvider mixerProvider2 = (MixerProvider)mixerProviders.get(k);
            Mixer.Info[] mixerInfo2 = mixerProvider2.getMixerInfo();
            for (int l = 0; l < mixerInfo2.length; ++l) {
                try {
                    Mixer mixer2 = mixerProvider2.getMixer(mixerInfo2[l]);
                    if (isAppropriateMixer(mixer2, info, false)) {
                        return mixer2.getLine(info);
                    }
                }
                catch (LineUnavailableException ex6) {
                    ex = ex6;
                }
                catch (IllegalArgumentException ex7) {}
            }
        }
        if (ex != null) {
            throw ex;
        }
        throw new IllegalArgumentException("No line matching " + info.toString() + " is supported.");
    }
    
    public static Clip getClip() throws LineUnavailableException {
        return (Clip)getLine(new DataLine.Info(Clip.class, new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, -1.0f, 16, 2, 4, -1.0f, true)));
    }
    
    public static Clip getClip(Mixer.Info info) throws LineUnavailableException {
        return (Clip)getMixer(info).getLine(new DataLine.Info(Clip.class, new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, -1.0f, 16, 2, 4, -1.0f, true)));
    }
    
    public static SourceDataLine getSourceDataLine(AudioFormat audioFormat) throws LineUnavailableException {
        return (SourceDataLine)getLine(new DataLine.Info(SourceDataLine.class, audioFormat));
    }
    
    public static SourceDataLine getSourceDataLine(AudioFormat audioFormat, Mixer.Info info) throws LineUnavailableException {
        return (SourceDataLine)getMixer(info).getLine(new DataLine.Info(SourceDataLine.class, audioFormat));
    }
    
    public static TargetDataLine getTargetDataLine(AudioFormat audioFormat) throws LineUnavailableException {
        return (TargetDataLine)getLine(new DataLine.Info(TargetDataLine.class, audioFormat));
    }
    
    public static TargetDataLine getTargetDataLine(AudioFormat audioFormat, Mixer.Info info) throws LineUnavailableException {
        return (TargetDataLine)getMixer(info).getLine(new DataLine.Info(TargetDataLine.class, audioFormat));
    }
    
    public static AudioFormat.Encoding[] getTargetEncodings(AudioFormat.Encoding sourceEncoding) {
        List<Object> formatConversionProviders = getFormatConversionProviders();
        Vector<Object> vector = new Vector();
        for (int i = 0; i < formatConversionProviders.size(); ++i) {
            FormatConversionProvider formatConversionProvider = (FormatConversionProvider)formatConversionProviders.get(i);
            if (formatConversionProvider.isSourceEncodingSupported(sourceEncoding)) {
                AudioFormat.Encoding[] targetEncodings = formatConversionProvider.getTargetEncodings();
                for (int j = 0; j < targetEncodings.length; ++j) {
                    vector.addElement(targetEncodings[j]);
                }
            }
        }
        return (AudioFormat.Encoding[])(AudioFormat.Encoding[])vector.toArray(new AudioFormat.Encoding[0]);
    }
    
    public static AudioFormat.Encoding[] getTargetEncodings(AudioFormat audioFormat) {
        List<Object> formatConversionProviders = getFormatConversionProviders();
        Vector<AudioFormat.Encoding> vector = new Vector();
        int n = 0;
        int n2 = 0;
        for (int i = 0; i < formatConversionProviders.size(); ++i) {
            AudioFormat.Encoding[] targetEncodings = ((FormatConversionProvider)formatConversionProviders.get(i)).getTargetEncodings(audioFormat);
            n += targetEncodings.length;
            vector.addElement(targetEncodings);
        }
        AudioFormat.Encoding[] array = new AudioFormat.Encoding[n];
        for (int j = 0; j < vector.size(); ++j) {
            AudioFormat.Encoding[] array2 = (AudioFormat.Encoding[])(AudioFormat.Encoding[])vector.get(j);
            for (int k = 0; k < array2.length; ++k) {
                array[n2++] = array2[k];
            }
        }
        return array;
    }
    
    public static boolean isConversionSupported(AudioFormat.Encoding targetEncoding, AudioFormat sourceFormat) {
        List<Object> formatConversionProviders = getFormatConversionProviders();
        int i = 0;
        while (i < formatConversionProviders.size()) {
            if (((FormatConversionProvider)formatConversionProviders.get(i)).isConversionSupported(targetEncoding, sourceFormat)) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    public static AudioInputStream getAudioInputStream(AudioFormat.Encoding encoding, AudioInputStream audioInputStream) {
        List<Object> formatConversionProviders = getFormatConversionProviders();
        int i = 0;
        while (i < formatConversionProviders.size()) {
            FormatConversionProvider formatConversionProvider = (FormatConversionProvider)formatConversionProviders.get(i);
            if (formatConversionProvider.isConversionSupported(encoding, audioInputStream.getFormat())) {
                return formatConversionProvider.getAudioInputStream(encoding, audioInputStream);
            }
            ++i;
        }
        throw new IllegalArgumentException("Unsupported conversion: " + encoding + " from " + audioInputStream.getFormat());
    }
    
    public static AudioFormat[] getTargetFormats(AudioFormat.Encoding encoding, AudioFormat audioFormat) {
        List<Object> formatConversionProviders = getFormatConversionProviders();
        Vector<AudioFormat> vector = new Vector();
        int n = 0;
        int n2 = 0;
        for (int i = 0; i < formatConversionProviders.size(); ++i) {
            AudioFormat[] targetFormats = ((FormatConversionProvider)formatConversionProviders.get(i)).getTargetFormats(encoding, audioFormat);
            n += targetFormats.length;
            vector.addElement(targetFormats);
        }
        AudioFormat[] array = new AudioFormat[n];
        for (int j = 0; j < vector.size(); ++j) {
            AudioFormat[] array2 = (AudioFormat[])(AudioFormat[])vector.get(j);
            for (int k = 0; k < array2.length; ++k) {
                array[n2++] = array2[k];
            }
        }
        return array;
    }
    
    public static boolean isConversionSupported(AudioFormat targetFormat, AudioFormat sourceFormat) {
        List<Object> formatConversionProviders = getFormatConversionProviders();
        int i = 0;
        while (i < formatConversionProviders.size()) {
            if (((FormatConversionProvider)formatConversionProviders.get(i)).isConversionSupported(targetFormat, sourceFormat)) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    public static AudioInputStream getAudioInputStream(AudioFormat audioFormat, AudioInputStream audioInputStream) {
        if (audioInputStream.getFormat().matches(audioFormat)) {
            return audioInputStream;
        }
        List<Object> formatConversionProviders = getFormatConversionProviders();
        int i = 0;
        while (i < formatConversionProviders.size()) {
            FormatConversionProvider formatConversionProvider = (FormatConversionProvider)formatConversionProviders.get(i);
            if (formatConversionProvider.isConversionSupported(audioFormat, audioInputStream.getFormat())) {
                return formatConversionProvider.getAudioInputStream(audioFormat, audioInputStream);
            }
            ++i;
        }
        throw new IllegalArgumentException("Unsupported conversion: " + audioFormat + " from " + audioInputStream.getFormat());
    }
    
    public static AudioFileFormat getAudioFileFormat(InputStream inputStream) throws UnsupportedAudioFileException, IOException {
        List<Object> audioFileReaders = getAudioFileReaders();
        AudioFileFormat audioFileFormat = null;
        int i = 0;
        while (i < audioFileReaders.size()) {
            AudioFileReader audioFileReader = (AudioFileReader)audioFileReaders.get(i);
            try {
                audioFileFormat = audioFileReader.getAudioFileFormat(inputStream);
                break;
            }
            catch (UnsupportedAudioFileException ex) {
                ++i;
            }
        }
        if (audioFileFormat == null) {
            throw new UnsupportedAudioFileException("file is not a supported file type");
        }
        return audioFileFormat;
    }
    
    public static AudioFileFormat getAudioFileFormat(URL url) throws UnsupportedAudioFileException, IOException {
        List<Object> audioFileReaders = getAudioFileReaders();
        AudioFileFormat audioFileFormat = null;
        int i = 0;
        while (i < audioFileReaders.size()) {
            AudioFileReader audioFileReader = (AudioFileReader)audioFileReaders.get(i);
            try {
                audioFileFormat = audioFileReader.getAudioFileFormat(url);
                break;
            }
            catch (UnsupportedAudioFileException ex) {
                ++i;
            }
        }
        if (audioFileFormat == null) {
            throw new UnsupportedAudioFileException("file is not a supported file type");
        }
        return audioFileFormat;
    }
    
    public static AudioFileFormat getAudioFileFormat(File file) throws UnsupportedAudioFileException, IOException {
        List<Object> audioFileReaders = getAudioFileReaders();
        AudioFileFormat audioFileFormat = null;
        int i = 0;
        while (i < audioFileReaders.size()) {
            AudioFileReader audioFileReader = (AudioFileReader)audioFileReaders.get(i);
            try {
                audioFileFormat = audioFileReader.getAudioFileFormat(file);
                break;
            }
            catch (UnsupportedAudioFileException ex) {
                ++i;
            }
        }
        if (audioFileFormat == null) {
            throw new UnsupportedAudioFileException("file is not a supported file type");
        }
        return audioFileFormat;
    }
    
    public static AudioInputStream getAudioInputStream(InputStream inputStream) throws UnsupportedAudioFileException, IOException {
        List<Object> audioFileReaders = getAudioFileReaders();
        AudioInputStream audioInputStream = null;
        int i = 0;
        while (i < audioFileReaders.size()) {
            AudioFileReader audioFileReader = (AudioFileReader)audioFileReaders.get(i);
            try {
                audioInputStream = audioFileReader.getAudioInputStream(inputStream);
                break;
            }
            catch (UnsupportedAudioFileException ex) {
                ++i;
            }
        }
        if (audioInputStream == null) {
            throw new UnsupportedAudioFileException("could not get audio input stream from input stream");
        }
        return audioInputStream;
    }
    
    public static AudioInputStream getAudioInputStream(URL url) throws UnsupportedAudioFileException, IOException {
        List<Object> audioFileReaders = getAudioFileReaders();
        AudioInputStream audioInputStream = null;
        int i = 0;
        while (i < audioFileReaders.size()) {
            AudioFileReader audioFileReader = (AudioFileReader)audioFileReaders.get(i);
            try {
                audioInputStream = audioFileReader.getAudioInputStream(url);
                break;
            }
            catch (UnsupportedAudioFileException ex) {
                ++i;
            }
        }
        if (audioInputStream == null) {
            throw new UnsupportedAudioFileException("could not get audio input stream from input URL");
        }
        return audioInputStream;
    }
    
    public static AudioInputStream getAudioInputStream(File file) throws UnsupportedAudioFileException, IOException {
        List<Object> audioFileReaders = getAudioFileReaders();
        AudioInputStream audioInputStream = null;
        int i = 0;
        while (i < audioFileReaders.size()) {
            AudioFileReader audioFileReader = (AudioFileReader)audioFileReaders.get(i);
            try {
                audioInputStream = audioFileReader.getAudioInputStream(file);
                break;
            }
            catch (UnsupportedAudioFileException ex) {
                ++i;
            }
        }
        if (audioInputStream == null) {
            throw new UnsupportedAudioFileException("could not get audio input stream from input file");
        }
        return audioInputStream;
    }
    
    public static AudioFileFormat.Type[] getAudioFileTypes() {
        List<Object> audioFileWriters = getAudioFileWriters();
        HashSet<Object> set = new HashSet();
        for (int i = 0; i < audioFileWriters.size(); ++i) {
            AudioFileFormat.Type[] audioFileTypes = ((AudioFileWriter)audioFileWriters.get(i)).getAudioFileTypes();
            for (int j = 0; j < audioFileTypes.length; ++j) {
                set.add(audioFileTypes[j]);
            }
        }
        return (AudioFileFormat.Type[])(AudioFileFormat.Type[])set.toArray(new AudioFileFormat.Type[0]);
    }
    
    public static boolean isFileTypeSupported(AudioFileFormat.Type fileType) {
        List<Object> audioFileWriters = getAudioFileWriters();
        int i = 0;
        while (i < audioFileWriters.size()) {
            if (((AudioFileWriter)audioFileWriters.get(i)).isFileTypeSupported(fileType)) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    public static AudioFileFormat.Type[] getAudioFileTypes(AudioInputStream audioInputStream) {
        List<Object> audioFileWriters = getAudioFileWriters();
        HashSet<Object> set = new HashSet();
        for (int i = 0; i < audioFileWriters.size(); ++i) {
            AudioFileFormat.Type[] audioFileTypes = ((AudioFileWriter)audioFileWriters.get(i)).getAudioFileTypes(audioInputStream);
            for (int j = 0; j < audioFileTypes.length; ++j) {
                set.add(audioFileTypes[j]);
            }
        }
        return (AudioFileFormat.Type[])(AudioFileFormat.Type[])set.toArray(new AudioFileFormat.Type[0]);
    }
    
    public static boolean isFileTypeSupported(AudioFileFormat.Type fileType, AudioInputStream stream) {
        List<Object> audioFileWriters = getAudioFileWriters();
        int i = 0;
        while (i < audioFileWriters.size()) {
            if (((AudioFileWriter)audioFileWriters.get(i)).isFileTypeSupported(fileType, stream)) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    public static int write(AudioInputStream audioInputStream, AudioFileFormat.Type obj, OutputStream outputStream) throws IOException {
        List<Object> audioFileWriters = getAudioFileWriters();
        int write = 0;
        boolean n = false;
        int i = 0;
        while (i < audioFileWriters.size()) {
            AudioFileWriter audioFileWriter = (AudioFileWriter)audioFileWriters.get(i);
            try {
                write = audioFileWriter.write(audioInputStream, obj, outputStream);
                n = true;
                break;
            }
            catch (IllegalArgumentException ex) {
                ++i;
            }
        }
        if (n == 0) {
            throw new IllegalArgumentException("could not write audio file: file type not supported: " + obj);
        }
        return write;
    }
    
    public static int write(AudioInputStream audioInputStream, AudioFileFormat.Type obj, File file) throws IOException {
        List<Object> audioFileWriters = getAudioFileWriters();
        int write = 0;
        boolean n = false;
        int i = 0;
        while (i < audioFileWriters.size()) {
            AudioFileWriter audioFileWriter = (AudioFileWriter)audioFileWriters.get(i);
            try {
                write = audioFileWriter.write(audioInputStream, obj, file);
                n = true;
                break;
            }
            catch (IllegalArgumentException ex) {
                ++i;
            }
        }
        if (n == 0) {
            throw new IllegalArgumentException("could not write audio file: file type not supported: " + obj);
        }
        return write;
    }
    
    private static List getMixerProviders() {
        return getProviders(MixerProvider.class);
    }
    
    private static List getFormatConversionProviders() {
        return getProviders(FormatConversionProvider.class);
    }
    
    private static List getAudioFileReaders() {
        return getProviders(AudioFileReader.class);
    }
    
    private static List getAudioFileWriters() {
        return getProviders(AudioFileWriter.class);
    }
    
    private static Mixer getDefaultMixer(List list, Line.Info info) {
        Class<Object> lineClass = info.getLineClass();
        String defaultProviderClassName = JDK13Services.getDefaultProviderClassName(lineClass);
        String defaultInstanceName = JDK13Services.getDefaultInstanceName(lineClass);
        if (defaultProviderClassName != null) {
            MixerProvider namedProvider = getNamedProvider(defaultProviderClassName, list);
            if (namedProvider != null) {
                if (defaultInstanceName != null) {
                    Mixer namedMixer = getNamedMixer(defaultInstanceName, namedProvider, info);
                    if (namedMixer != null) {
                        return namedMixer;
                    }
                }
                else {
                    Mixer firstMixer = getFirstMixer(namedProvider, info, false);
                    if (firstMixer != null) {
                        return firstMixer;
                    }
                }
            }
        }
        if (defaultInstanceName != null) {
            Mixer namedMixer2 = getNamedMixer(defaultInstanceName, list, info);
            if (namedMixer2 != null) {
                return namedMixer2;
            }
        }
        return null;
    }
    
    private static MixerProvider getNamedProvider(String anObject, List list) {
        int i = 0;
        while (i < list.size()) {
            MixerProvider mixerProvider = (MixerProvider)list.get(i);
            if (mixerProvider.getClass().getName().equals(anObject)) {
                return mixerProvider;
            }
            ++i;
        }
        return null;
    }
    
    private static Mixer getNamedMixer(String anObject, MixerProvider mixerProvider, Line.Info info) {
        Mixer.Info[] mixerInfo = mixerProvider.getMixerInfo();
        for (int i = 0; i < mixerInfo.length; ++i) {
            if (mixerInfo[i].getName().equals(anObject)) {
                Mixer mixer = mixerProvider.getMixer(mixerInfo[i]);
                if (isAppropriateMixer(mixer, info, false)) {
                    return mixer;
                }
            }
        }
        return null;
    }
    
    private static Mixer getNamedMixer(String s, List list, Line.Info info) {
        int i = 0;
        while (i < list.size()) {
            Mixer namedMixer = getNamedMixer(s, (MixerProvider)list.get(i), info);
            if (namedMixer != null) {
                return namedMixer;
            }
            ++i;
        }
        return null;
    }
    
    private static Mixer getFirstMixer(MixerProvider mixerProvider, Line.Info info, boolean b) {
        Mixer.Info[] mixerInfo = mixerProvider.getMixerInfo();
        int i = 0;
        while (i < mixerInfo.length) {
            Mixer mixer = mixerProvider.getMixer(mixerInfo[i]);
            if (isAppropriateMixer(mixer, info, b)) {
                return mixer;
            }
            ++i;
        }
        return null;
    }
    
    private static boolean isAppropriateMixer(Mixer mixer, Line.Info info, boolean b) {
        if (!mixer.isLineSupported(info)) {
            return false;
        }
        Class<Object> lineClass = info.getLineClass();
        if (b && (SourceDataLine.class.isAssignableFrom(lineClass) || Clip.class.isAssignableFrom(lineClass))) {
            int maxLines = mixer.getMaxLines(info);
            return maxLines == -1 || maxLines > 1;
        }
        return true;
    }
    
    private static List getMixerInfoList() {
        return getMixerInfoList(getMixerProviders());
    }
    
    private static List getMixerInfoList(List list) {
        ArrayList<Object> list2 = new ArrayList();
        for (int i = 0; i < list.size(); ++i) {
            Mixer.Info[] array = (Mixer.Info[])((MixerProvider)list.get(i)).getMixerInfo();
            for (int j = 0; j < array.length; ++j) {
                list2.add(array[j]);
            }
        }
        return list2;
    }
    
    private static List getProviders(Class serviceClass) {
        return JDK13Services.getProviders(serviceClass);
    }
}
